function nameOfKeyPressed = oms_awaitkeypress(delay_s,~,requiredKey,awaitPreviousKeyRelease)
% Provides a delay that can be skipped over by pressing a specified key
% Can also be used to force a delay that *cannot* be skipped
% Version 2.1 - 2021 by Matt J Dunn

% requiredKey should be a string or a cell array of strings; each are
% possible key names that will allow the program to continue
% The function returns nameOfKeyPressed, which is a string with
% the name of the key that ultimately allowed the function to end, or, the
% last key that was pressed if the function timed out

if ~exist('delay_s','var') || isempty(delay_s) || isnan(delay_s)
    delay_s = inf;
end
if ~exist('requiredKey','var') || isempty(requiredKey)
    requiredKey = 'any'; % 'any' is a special case that will allow anything
end
if ~exist('awaitPreviousKeyRelease','var') || isempty(awaitPreviousKeyRelease)
    awaitPreviousKeyRelease = true;
end

if awaitPreviousKeyRelease
    KbReleaseWait; % await release of any pre-existing key presses
end

nameOfKeyPressed = ''; % initialise as blank string; default if nothing is pressed

if delay_s < 0
    return
end

stopTime = GetSecs + delay_s; % time to stop waiting is decided
while GetSecs < stopTime
    [keyIsDown,~,keyCode]= KbCheck; % check the keyboard
    if keyIsDown ~= 0
        nameOfKeyPressed = KbName(keyCode); % convert to name
        if ischar(nameOfKeyPressed) && (any(strcmpi(requiredKey,'any')) || any(strcmpi(requiredKey,nameOfKeyPressed))) % if the appropriate key was pressed (and only one key was pressed)
                return % leave the function
        end
    end
end
