function [fixationPosition_px,fixationTargetRect] = oms_drawfixation(window,ScreenInfo,diameter_deg,position_deg,fixationColour_rgb,~,delayFlip,fixationType,backgroundColour_rgb)
% Code written 2015-2019 by Matt J Dunn

if ~exist('diameter_deg','var') || isempty(diameter_deg)
    diameter_deg = 1;
end
if ~exist('position_deg','var') || isempty(position_deg)
    position_deg = [0,0];
end
if ~exist('fixationColour_rgb','var') || isempty(fixationColour_rgb)
    fixationColour_rgb = 0;
end
if ~exist('delayFlip','var') || isempty(delayFlip)
    delayFlip = false; % if true, do not flip the screen in this function
end
if ~exist('fixationType','var') || isempty(fixationType)
    fixationType = 'standard'; % set to 'standard', 'cross', 'worth', 'circle', 'square' or 'animal'
end
if ~exist('backgroundColour_rgb','var') || isempty(backgroundColour_rgb)
    backgroundColour_rgb = ScreenInfo.backgroundColour_rgb; % pull from ScreenInfo
end

totalSize_px = oms_convert_degrees2px(diameter_deg,ScreenInfo); % convert from degrees to px
fixationPosition_px(1) = oms_convert_degrees2px(position_deg(1),ScreenInfo) + ScreenInfo.width_px/2; % convert from degrees to screen px
fixationPosition_px(2) = oms_convert_degrees2px(-position_deg(2),ScreenInfo) + ScreenInfo.height_px/2; % convert from degrees to screen px (note that Y axis data must be negative due to monitor coordinates)

if length(totalSize_px) == 1
    totalSize_px = [totalSize_px,totalSize_px]; % if only one size is given, we assume the user wants equal dimensions
end
fixationTargetRect = [0,0,totalSize_px(1),totalSize_px(2)];
fixationTargetRect = CenterRectOnPointd(fixationTargetRect,fixationPosition_px(1),fixationPosition_px(2));

switch fixationType
    case 'standard' % based on parameters described in {Thaler, Schutz, Goodale and Gegenfurtner (2013) What is the best fixation target? The effect of target shape on stability of fixational eye movements. Vision research 76: 31-42}
        crossWidth_px = oms_convert_degrees2px(diameter_deg,ScreenInfo)/3; % convert from degrees to px
        
        % Prepare outer circle:
        Screen('FillOval',window,fixationColour_rgb,[fixationPosition_px(1)-totalSize_px(1)/2,fixationPosition_px(2)-totalSize_px(2)/2,fixationPosition_px(1)+totalSize_px(1)/2,fixationPosition_px(2)+totalSize_px(2)/2],totalSize_px(2));
        
        % Prepare cross:
        Cross.horizontal = [0,0,totalSize_px(1),crossWidth_px]; % prepare horizontal bar of cross
        Cross.horizontalCentred = CenterRectOnPointd(Cross.horizontal,fixationPosition_px(1),fixationPosition_px(2)); % center the rect on the position
        Screen('FillRect',window,backgroundColour_rgb,Cross.horizontalCentred);
        Cross.vertical = [0,0,crossWidth_px,totalSize_px(2)]; % prepare vertical bar of cross
        Cross.verticalCentred = CenterRectOnPointd(Cross.vertical,fixationPosition_px(1),fixationPosition_px(2)); % center the rect on the position
        Screen('FillRect',window,backgroundColour_rgb,Cross.verticalCentred);
        
        % Prepare inner circle:
        Screen('FillOval',window,fixationColour_rgb,[fixationPosition_px(1)-crossWidth_px/2,fixationPosition_px(2)-crossWidth_px/2,fixationPosition_px(1)+crossWidth_px/2,fixationPosition_px(2)+crossWidth_px/2],totalSize_px(1));
end

if ~delayFlip
    Screen('Flip',window); % actually display the screen
end
