function [newSampleAvailable,re_xPosition,re_yPosition,le_xPosition,le_yPosition,re_pupil,le_pupil,re_distance_mm,le_distance_mm,time_ms] = oms_eyetracker_poll(tracker,ScreenInfo,~)
% Poll eyetracker and update eyetrace
% This function is called by oms_eyetracker_getgazedata_latestsample and oms_eyetracker_getgazedata_sincepoll. Do not call it directly unless you intend to flush/update an on-board eyetrace (e.g. Tobii/GazePoint).

if ~exist('tracker','var') || isempty(tracker)
    tracker = oms_eyetracker_getconnectedtracker;
end

[re_xPosition,re_yPosition,le_xPosition,le_yPosition,re_pupil,le_pupil,re_distance_mm,le_distance_mm,time_ms] = deal(nan); % init (in case nothing returned)

newSampleAvailable = false; % init
switch tracker
    case 'eyelink'
        if Eyelink('NewFloatSampleAvailable') == 1 % is there a new sample available in the buffer?
            newSampleAvailable = true;
            sample = Eyelink('NewestFloatSample');
            switch dataType % grab the appropriate data depending on which type we want
                case 'gaze'
                    xPosition = sample.gx;
                    yPosition = sample.gy;
                case 'pupil'
                    xPosition = sample.px;
                    yPosition = sample.py;
                case 'href'
                    xPosition = sample.hx;
                    yPosition = sample.hy;
                case 'raw'
                    % TODO
            end
            
            xPosition(xPosition == -32768) = NaN; % -32768 is the number reported if the eye isn't found
            yPosition(yPosition == -32768) = NaN; % -32768 is the number reported if the eye isn't found
            
            re_xPosition = xPosition(2);
            re_yPosition = yPosition(2);
            le_xPosition = xPosition(1);
            le_yPosition = yPosition(1);
            
            time_ms = sample.time;
        else
            newSampleAvailable = false;
        end
    case 'none'
        newSampleAvailable = true; % always return true
        [xMouse,yMouse] = GetMouse;
        [re_xPosition,le_xPosition] = deal(xMouse * 2); % *2 to account for retina display issue. So mouse simulation will only work properly on Mac at present.
        [re_yPosition,le_yPosition] = deal(yMouse * 2); % *2 to account for retina display issue. So mouse simulation will only work properly on Mac at present.
        time_ms = round(GetSecs * 1000); % get time in ms
end
