function finalFileName = oms_eyetracker_shutdown(tempEyeTraceFileName,finalFileName,~,tracker,collectData,~)
% Receive data from eyetracker and save
% Written in 2018 by Matt J Dunn

if ~exist('finalFileName','var') || isempty(finalFileName)
    finalFileName = 'eyetrackingdata';
end
if ~exist('collectData','var') || isempty(collectData)
    collectData = true;
end
if ~exist('tracker','var') || isempty(tracker)
    tracker = oms_eyetracker_getconnectedtracker; % find which eye tracker is connected
end

[finalFileFolder,finalFileName] = fileparts(finalFileName);

[~,~] = mkdir(finalFileFolder); % create the folder

%% Add file extension
switch tracker
    case 'eyelink'
        finalFileExtension = '.edf';
    otherwise
        finalFileExtension = '.csv';
end

finalFileName = fullfile(finalFileFolder,[finalFileName,finalFileExtension]); % recalculate final file name, taking into account the file extension

%% Download data from the eyetracker and save
switch tracker
    case 'eyelink'
        % TODO: Check tracking is currently underway, and only stop if it isn't
        Eyelink('command','set_idle_mode');
        WaitSecs('YieldSecs',0.5); % give the EL1000 a chance to catch up
        Eyelink('CloseFile');
        if collectData
            % Download data file`
            fprintf('\nReceiving data file');
            try status = Eyelink('ReceiveFile'); end
            if exist('status','var') && status > 0
                movefile(tempEyeTraceFileName,finalFileName);
                fprintf('\nData file saved to %s',[finalFileName,finalFileExtension]);
            else
                warning('Problem receiving data file'); % if data transfer fails, let us know
            end
        end
        Eyelink('Shutdown'); % switch off EL1000
    case 'none'
        % pass
    otherwise
        fprintf('\nUnable to find an active eye tracker to shut down')
end
