function [tempEyeTraceFileName,tracker,ScreenInfo,window,eyeToTrack,window_overlay] = oms_eyetracker_startup(tracker,calibrate,startRecordingImmediately,~,screenName,backgroundColour_rgb,~,includeWaitScreen,eyeToTrack)

if ~exist('tracker','var') || isempty(tracker)
    tracker = oms_eyetracker_getavailabletracker;
end
if ~exist('calibrate','var') || isempty(calibrate)
    calibrate = true;
end
if ~exist('startRecordingImmediately','var') || isempty(startRecordingImmediately)
    startRecordingImmediately = true;
end
if ~exist('screenName','var') || isempty(screenName)
    screenName = [];
end
if ~exist('backgroundColour_rgb','var') || isempty(backgroundColour_rgb)
    backgroundColour_rgb = [];
end
if ~exist('includeWaitScreen','var') || isempty(includeWaitScreen)
    includeWaitScreen = false;
end

clocknow = clock; % grab the date and time, as this will form our temporary filename
uniqueTimeDateCode = [num2str(clocknow(2),'%02.f'),num2str(clocknow(3),'%02.f'),num2str(clocknow(4),'%02.f'),num2str(clocknow(5),'%02.f')]; % define the temporary name of our file (must be 8 characters or less; we choose a name unique to the time)

[ScreenInfo,window,window_overlay] = oms_screen_startup(screenName,[],backgroundColour_rgb,includeWaitScreen);

switch tracker
    case 'eyelink'
        Eyelink('Initialize','PsychEyelinkDispatchCallback'); % initialise EL1000 with callbacks to PsychToolbox
        EYETRACKERHANDLE = EyelinkInitDefaults(window); % initialise EL1000 default config
        EYETRACKERHANDLE.backgroundcolour = backgroundColour_rgb; % change background colour to whatever is selected by user
        EYETRACKERHANDLE.foregroundcolour = [0.3,0,0]; % change foreground colour to red
        EYETRACKERHANDLE.msgfontcolour = [0.3,0,0]; % change font colour to red
        EYETRACKERHANDLE.imgtitlecolour = [0.3,0,0]; % change image title colour to red
        EYETRACKERHANDLE.calibrationtargetcolour = [0.6,0,0]; % change calibration target colour to red
        EYETRACKERHANDLE.calibrationtargetsize = 2;
        EyelinkUpdateDefaults(EYETRACKERHANDLE); % update EL1000 setup screen
        Eyelink('command', 'auto_calibration_messages = YES'); % ensure calibration/validation info is printed into the messagestream
        Eyelink('command',['screen_physical_coords = ',num2str(-floor(ScreenInfo.width_mm/2)),',',num2str(floor(ScreenInfo.height_mm/2)),',',num2str(floor(ScreenInfo.width_mm/2)),',',num2str(-floor(ScreenInfo.height_mm/2))]) % mm from screen centre to left, top, right and bottom % TODO: Incorporate automation to this
        Eyelink('command',['screen_distance ',num2str(round(ScreenInfo.distance_mm)),' ',num2str(round(ScreenInfo.distance_mm))]) % mm from eye to screen top and bottom, retreived by querying ScreenInfo. Note that we assume this is the same for both top and bottom; i.e. that the eyes are in the vertical middle of the screen and there is no tilt
        
        maxEyeLinkRange_deg = 25; % the maximum range of eye position (±) that the EyeLink is comfortably capable of tracking. The manual states ±30° but in reality we can't expect to calibrate at these positions, so use ±25°
        if max(ScreenInfo.width_deg,ScreenInfo.height_deg)/2 * 0.85 > maxEyeLinkRange_deg % if the screen is too large for us to reliably calibrate 85% of the maximum screen dimension
            percentageOfScreenToCalibrate = maxEyeLinkRange_deg / (max(ScreenInfo.width_deg,ScreenInfo.height_deg)/2); % calculate, as a decimal (0 to 1) the fraction of the screen that we can comfortably calibrate
        else
            percentageOfScreenToCalibrate = 0.85; % by default, we calibrate 85% of the screen area
        end
        
        Eyelink('command',['calibration_area_proportion ',num2str(round(percentageOfScreenToCalibrate,2)),' ',num2str(round(percentageOfScreenToCalibrate,2))]); % set the portion of the screen to use for calibration
        Eyelink('command',['vaidation_area_proportion ',num2str(round(percentageOfScreenToCalibrate,2)),' ',num2str(round(percentageOfScreenToCalibrate,2))]); % set the portion of the screen to use for validation
        Eyelink('command','calibration_type = HV5'); % set calibration type
        tempEyeTraceFileName = [uniqueTimeDateCode,'.edf'];
        if calibrate
            EyelinkDoTrackerSetup(EYETRACKERHANDLE); % begin camera setup procedure for any last minute adjustments % TODO: Delay appears to be in here...
        end
        Eyelink('command','link_sample_data = LEFT,RIGHT,GAZE,AREA,GAZERES,HREF,PUPIL,STATUS,INPUT'); % define which data to record to EDF file
        Eyelink('Openfile',tempEyeTraceFileName); % open the EDF file
        fprintf('\nOpening temporary recording file ''%s''\n',tempEyeTraceFileName)
        [windowWidth,windowHeight] = Screen('WindowSize',window); % grab window properties
        Eyelink('command','screen_pixel_coords = %ld %ld %ld %ld',0,0,windowWidth-1,windowHeight-1); % tell the EL1000 what coordinates we are working with
        Eyelink('message','DISPLAY_COORDS %ld %ld %ld %ld',0,0,windowWidth-1,windowHeight-1); % "
        Screen('FillRect',window,backgroundColour_rgb);
        Screen('Flip',window); % flip to remove screen content
        Eyelink('command','set_idle_mode'); % set EL1000 idle, ready for next trial
        WaitSecs('YieldSecs',0.05); % wait a moment to let the EL1000 catch up
    otherwise
        tempEyeTraceFileName = '';
end
if startRecordingImmediately
    oms_eyetracker_startrecording(tracker);
end
