function fileName = oms_generatecsv(report,fileName,header,~)
% Simple function to create a CSV report. Supply a 2D array, filename and a string of header names, separated by commas, in quotes
% Created by Matt J Dunn 2014-2022

if ~exist('fileName','var') || isempty(fileName)
    fileName = 'output.csv';
elseif ~strcmp(fileName(end-3:end),'.csv') % if fileName doesn't end with '.csv'
    fileName = [fileName,'.csv']; % add it
end
if ~exist('header','var')
    header = [];
end

filePath = fileparts(fileName); % check for a file path in the desired output filename

if ~isempty(filePath) % if a path has been specified
    [~,~] = mkdir(filePath); % create it
end

if isnumeric(report)
    report = num2cell(report); % convert numeric array to cell array
end

report(all(cellfun(@isempty,report),2),:) = []; % delete any blank rows

fid = fopen(fileName,'w+');
if ~isempty(header)
    fprintf(fid,strcat(header,'\n')); % print header to file (note this is allowed to contain multiple lines)
end

cellCells = find(cellfun(@iscell,report)); % find cells that contain another cell

for i = 1:length(cellCells) % for each cellCell    
    report(cellCells(i)) = {report{cellCells(i)}{:}}; % extract the contents of the cell
end

%% Query all the cells to work out which data we need to put in our report
numericCells = find(cellfun(@isnumeric,report)); % which cells contain a number?
nanCells = isnan([report{numericCells}]); % of these, which are NaNs?

logicalCells = find(cellfun(@islogical,report)); % which cells are boolean?
trueCells = [report{logicalCells}]; % of these, which are TRUE?

charCells = find(cellfun(@ischar,report)); % which cells already contain text?

%% Build a cell array in which all data are stored as strings
strArray = cell(size(report)); % init

strArray(numericCells) = cellfun(@num2str,report(numericCells),'UniformOutput',false); % put numbers in as text % TODO: This line takes absolutely ages to execute. Make it more efficient
strArray(numericCells(nanCells)) = {''}; % set NaNs to ''
strArray(logicalCells(trueCells)) = {'TRUE'}; % convert 1s to TRUE
strArray(logicalCells(~trueCells)) = {'FALSE'}; % convert 0s to FALSE
for i = 1:length(charCells) % for each charCell
    report(charCells(i)) = strtrim(report(charCells(i))); % trim any whitespace from beginning and end of string
    if contains(report(charCells(i)),',') % if the value contains a comma, we need to print the output in quotes for the CSV file
        strArray(charCells(i)) = {['"',report{charCells(i)},'"']}; % place in double quotes
    else
        strArray(charCells(i)) = report(charCells(i)); % otherwise, just transcribe like-for-like
    end
end

stringPrintFormat = repmat('%s,',1,size(strArray,2));
stringPrintFormat(end:end+1) = '\n'; % replace last comma with newline character

for i = 1:size(strArray,1) % for each row
    fprintf(fid,stringPrintFormat,strArray{i,:}); % print row
end

fclose(fid);
fprintf('\nGenerated file ''%s''',fileName)
