function [ScreenInfo,allScreenNames] = oms_getscreeninfo(screenName)

% TODO: Also retrieve whether or not an 'extended desktop' mode is used. If it is, we need to know, as ScreenInfo.width_px may be wrong

if ~exist('screenName','var') || isempty(screenName)
    screenName = [];
end

if isstruct(screenName) % if a struct is supplied...
    ScreenInfo = screenName; % ...then we assume this contains everything we need to know already, and don't run any further with this function
    return
end

fid = fopen('screens.csv');
screenPreferences = textscan(fid,'%s%s%f%f%f%f%f%f%s%s%f%f%f%f%s%s','Delimiter',',','HeaderLines',1,'EmptyValue',NaN);
fclose(fid);
for i = [9,16] % columns 9 and 16 contain true/false values
    screenPreferences{i}(strcmp(screenPreferences{i},'TRUE')) = {true}; % convert 'TRUE' to 'true'
    screenPreferences{i}(strcmp(screenPreferences{i},'FALSE')) = {false}; % convert 'FALSE' to 'false'
end
allScreenNames = screenPreferences{1};

if length(screenPreferences{1}) == 1 % if there is only one screen name, this is a version of OMS that Matt has set up to work out-of-the-box (I will have already put the correct screen details in)
    screenName = screenPreferences{1}{1};
else
    if length(unique(allScreenNames)) < length(allScreenNames)
        if ~ispc
            system(['open ''',which('screens.csv'),'''']); % open screens.csv
        else % on a PC
            winopen(which('screens.csv')); % open screens.csv
        end
        error('Multiple screens with the same name defined. Please adjust your screens.csv file');
    end
end

screenIndex = find(strcmp(screenPreferences{1},screenName));

ScreenInfo.width_mm = screenPreferences{3}(screenIndex);
ScreenInfo.height_mm = screenPreferences{4}(screenIndex);
ScreenInfo.width_px = screenPreferences{5}(screenIndex);
ScreenInfo.height_px = screenPreferences{6}(screenIndex);
ScreenInfo.distance_mm = screenPreferences{7}(screenIndex);
ScreenInfo.hz = screenPreferences{8}(screenIndex);
ScreenInfo.mirror = cell2mat(screenPreferences{9}(screenIndex));
%     ScreenInfo.gammaTableName = char(screenPreferences{10}(screenIndex)); % use this line instead if any problems with gammaTables
ScreenInfo.gammaTableName = screenPreferences{10}{screenIndex};
ScreenInfo.stereoMode = screenPreferences{11}(screenIndex); % for more info on stereoModes, see "help StereoDemo"
ScreenInfo.portId = screenPreferences{12}(screenIndex);
ScreenInfo.minLuminance_cdm2 = screenPreferences{13}(screenIndex);
ScreenInfo.maxLuminance_cdm2 = screenPreferences{14}(screenIndex);
ScreenInfo.bitsSharpMode = screenPreferences{15}{screenIndex};
ScreenInfo.dummyMode = cell2mat(screenPreferences{16}(screenIndex));

ScreenInfo.name = screenName;

if isnan(ScreenInfo.height_mm) % if not supplied, we calculate it based on the screen aspect ratio
    ScreenInfo.height_mm = round(oms_convert_px2mm(ScreenInfo.height_px,ScreenInfo.width_mm,ScreenInfo.width_px)); % NOTE: this calculation assumes square pixels
end

ScreenInfo.rect = [0,0,ScreenInfo.width_px,ScreenInfo.height_px]; % this one is for convenience later on
ScreenInfo.aspectRatio = ScreenInfo.width_px/ScreenInfo.height_px;
ScreenInfo.width_deg = oms_convert_px2degrees(ScreenInfo.width_px/2,ScreenInfo) * 2; % NOTE: This assumes the observer is sat centrally. The conversion between px and deg is increasingly non-linear for large values, due to curvature of field (we are assuming a flat screen).
ScreenInfo.height_deg = oms_convert_px2degrees(ScreenInfo.height_px/2,ScreenInfo) * 2; % NOTE: this calculation assumes square pixels
ScreenInfo.aliasingLimit_cpd = ScreenInfo.width_px/ScreenInfo.width_deg/2; % NOTE: This is a rough approximation of the maximum spatial frequency that this monitor can display. It is not accurate since the relationship between pixels and degrees isn't linear. Avoid going close to this limit, since stimuli will fail to render properly

% Sanity check: ensure the selected screen has square pixels
physicalScreenAspectRatio = ScreenInfo.width_mm/ScreenInfo.height_mm;
if abs(physicalScreenAspectRatio - ScreenInfo.aspectRatio) > 0.1 % we allow a 0.1 error, given that the measurement of mm may not be perfect. But anything above this could cause real problems.
    warning(['Your screen resolution is not appropriate for the physical monitor dimensions! Pixels are not square - please change screen resolution. If you think you have chosen an appropriate resolution, please check the ''screens.csv'' file, located in required_files/oms/preferences. Your screen width is apparently ',num2str(ScreenInfo.width_mm),'mm, and height is ',num2str(ScreenInfo.height_mm),'mm. STIMULI WILL BE SHOWN IN THE WRONG POSITIONS: DO NOT IGNORE THIS WARNING!'])
end
