function [StaircaseObject,nextStep,reversal] = oms_staircase_update(StaircaseObject,correct)
% Function to keep track of whether to move up or down in a staircase procedure
% Written 2014-2021 by Matt J Dunn

reversal = false; % by default, this is not a reversal
nextStep = 0; % by default, the next step will be no change

if correct
    StaircaseObject.nCorrect = StaircaseObject.nCorrect + 1;
    StaircaseObject.nIncorrect = 0;
    if StaircaseObject.nCorrect == StaircaseObject.nUp % move up staircase
        StaircaseObject.nCorrect = 0; % reset
        nextStep = +1; % instruction to move up
        if StaircaseObject.direction == -1 % if we were previously descending...
            reversal = true;
            StaircaseObject.nReversals = StaircaseObject.nReversals + 1;
        end
        StaircaseObject.direction = +1; % note we are now ascending
    end
else
    StaircaseObject.nIncorrect = StaircaseObject.nIncorrect + 1;
    StaircaseObject.nCorrect = 0;
    if StaircaseObject.nIncorrect == StaircaseObject.nDown % move down staircase
        StaircaseObject.nIncorrect = 0; % reset
        nextStep = -1; % instruction to move down
        if StaircaseObject.direction == +1 % if we were previously ascending...
            reversal = true;
            StaircaseObject.nReversals = StaircaseObject.nReversals + 1;
        end
        StaircaseObject.direction = -1; % note we are now descending
    end
end
if reversal
    StaircaseObject.reversals(StaircaseObject.nReversals) = StaircaseObject.stimulusIntensity;
end
if StaircaseObject.nReversals >= StaircaseObject.nReversalsRequired
    StaircaseObject.stop = true;
end

% Adjust intensity for next stimulus presentation:
switch StaircaseObject.stimulusStepType
    case 'fixed'
        StaircaseObject.stimulusIntensity = StaircaseObject.stimulusIntensity + nextStep * StaircaseObject.stimulusStepSize;
    case 'percentage'
        if nextStep == +1
            StaircaseObject.stimulusIntensity = StaircaseObject.stimulusIntensity * (1 + StaircaseObject.stimulusStepSize);
        elseif nextStep == -1
            StaircaseObject.stimulusIntensity = StaircaseObject.stimulusIntensity - StaircaseObject.stimulusIntensity * StaircaseObject.stimulusStepSize;
        end
end
