function [escapePressed,reactionTimeRecord_sec,nFalsePositiveResponses,nFalseNegativeResponses,StimulusList_main,pausePressed,stimulusActuallyPresented,repeatTrial] = presentation(trialType,StimulusList_main,nPresentationsThisWave,window,ScreenInfo,stimulusDuration_ms,CentralFixationTarget,backgroundLuminance_AU,eye,gazeContingencyType,FixationReminderAnnulus,~,tracker,driftCorrectionOffset_deg,manualFixationOffset_deg,reactionTimeRecord_sec,nFalsePositiveResponses,nFalseNegativeResponses,stimulusIndex,startTimestamp,falsePositiveCatchTime_sec,dynamicIntervalRange_min_max_sec,pausePressed,additionToInterstimulusInterval_sec,startingIntensityAboveNormativeData_logarithmic,falsePOSfalseNEG_logIncrement,maximumLuminance_cdm2)
% Select a stimulus intensity and present it

escapePressed = false; % init

stimulusPosition_deg = StimulusList_main(stimulusIndex).position_deg; % grab the properties of this stimulus from the list

switch trialType
    case 'quest'
        stimulusIntensity = QuestQuantile(StimulusList_main(stimulusIndex).QuestObject(size([StimulusList_main(stimulusIndex).QuestObject],2))); % calculate intensity of stimulus for each presentation of the stimulus
    case 'first' % the first presentation at this location is always elevated in brightness. As for QUEST presentation above, except we elevate the brightness to make this one (hopefully) easy to see
        predictedThreshold_au = QuestMean(StimulusList_main(stimulusIndex).QuestObject(size([StimulusList_main(stimulusIndex).QuestObject],2))); % get the predicted threshold at this stimulus position
        predictedThreshold_cdm2 = convert_au2cdm2(predictedThreshold_au,ScreenInfo); % convert into cdm2
        stimulusIntensity_dB = log10(maximumLuminance_cdm2/predictedThreshold_cdm2)*10 - startingIntensityAboveNormativeData_logarithmic; % this converts predictedThreshold_cdm2 into the stimulus intensity in dB (with the required logarithmic addition to the starting intensities - they should be 15 dB brighter than the predicted thresholds) 
        stimulusIntensity_cdm2 = 10.^(log10(maximumLuminance_cdm2) - stimulusIntensity_dB/10); % convert back to cdm2
        stimulusIntensity = convert_cdm22au(stimulusIntensity_cdm2,ScreenInfo); % convert back to au
    case 'false positive' % this is a false positive trial
        predictedThreshold_au = QuestMean(StimulusList_main(stimulusIndex).QuestObject(size([StimulusList_main(stimulusIndex).QuestObject],2))); % get the predicted threshold at this stimulus position
        predictedThreshold_cdm2 = convert_au2cdm2(predictedThreshold_au,ScreenInfo); % convert to cdm2
        stimulusIntensity_dB = log10(maximumLuminance_cdm2/predictedThreshold_cdm2)*10 + falsePOSfalseNEG_logIncrement; % decrease brightness by falsePOSfalseNEG_logIncrement logarithmic steps to catch false positives
        stimulusIntensity_cdm2 = 10.^(log10(maximumLuminance_cdm2) - stimulusIntensity_dB/10); % convert back to cdm2
        stimulusIntensity = convert_cdm22au(stimulusIntensity_cdm2,ScreenInfo); % convert back to au
    case 'false negative' % this is a false negative trial
        predictedThreshold_au = QuestMean(StimulusList_main(stimulusIndex).QuestObject(size([StimulusList_main(stimulusIndex).QuestObject],2))); % get the predicted threshold at this stimulus position
        predictedThreshold_cdm2 = convert_au2cdm2(predictedThreshold_au,ScreenInfo); % convert to cdm2
        stimulusIntensity_dB = log10(maximumLuminance_cdm2/predictedThreshold_cdm2)*10 - falsePOSfalseNEG_logIncrement; % increase brightness by falsePOSfalseNEG_logIncrement logarithmic steps to catch false positives 
        stimulusIntensity_cdm2 = 10.^(log10(maximumLuminance_cdm2) - stimulusIntensity_dB/10); % convert back to cdm2 
        stimulusIntensity = convert_cdm22au(stimulusIntensity_cdm2,ScreenInfo); % convert back to au
end
if stimulusIntensity <= backgroundLuminance_AU % check the stimulus isn't below the luminance of the background...
    stimulusIntensity = backgroundLuminance_AU;
    if ~strcmp(trialType,'false positive') % as long as this isn't a false positive catch trial...
        warning('Stimulus attempted at background luminance! Going 0.01 cdm2 above...');
        stimulusIntensity = backgroundLuminance_AU + convert_cdm22au(0.01,ScreenInfo); % WORKAROUND: If stimulus is at background luminance, show 1cdm2 above background luminance
    end
end
if stimulusIntensity > 1 % check the stimulus isn't above the screen's maximum brightness...
    stimulusIntensity = 1; % ...if it is, cap it
    warning('Attempting to show a stimulus with luminance greater than the screen maximum'); % ... and display a warning
end

stimulusStartTimestamp = GetSecs - startTimestamp; % record the system time at the beginning of stimulus presentation. This will be stored in the StimulusList later, so long as the trial isn't repeated

if nPresentationsThisWave > 5 && ~pausePressed
    if all(isnan(reactionTimeRecord_sec(end-4:end))) % if there was no response to any of the last five presentations...
        interstimulusInterval_sec = dynamicIntervalRange_min_max_sec(2); % set to the maximum interstimulusInterval allowed
    else
        interstimulusInterval_sec = mean(reactionTimeRecord_sec(end-4:end),'omitnan') + additionToInterstimulusInterval_sec; % change interstimulusInterval dynamically based on the mean of the previous five reaction times, and add a fixed amount of time
        if interstimulusInterval_sec < dynamicIntervalRange_min_max_sec(1)
            interstimulusInterval_sec = dynamicIntervalRange_min_max_sec(1);
        end
    end
else
    interstimulusInterval_sec = dynamicIntervalRange_min_max_sec(2); % set to the maximum interstimulusInterval allowed
end

interFrameInterval = Screen('GetFlipInterval',window); % how long is one monitor refresh interval?

oms_eyetracker_sendmessage(['Presentation number ',num2str(nPresentationsThisWave)]) % send a message to eye tracker
stimulusActuallyPresented = false; % init as false. If anything appears on the screen at any time, then this changes to true

repeatTrial = false; % init as false. It will only change to true if something happens to trigger a repeat (e.g. a blink occurs)
KbReleaseWait; % wait for old keypresses to finish
[~,~,gazePosition_deg] = oms_eyetracker_getgazedata_latestsample(eye,tracker,ScreenInfo);
gazePosition_deg = gazePosition_deg + driftCorrectionOffset_deg;
if strcmp(gazeContingencyType,'Continuous gaze-contingency (stimulus moving)') && norm(gazePosition_deg) > FixationReminderAnnulus.tolerance_deg || any(isnan(gazePosition_deg)) % check participant is fixating near screen center and bring them back if not (only if this is a gaze-contingent paradigm)
    while norm(gazePosition_deg) > FixationReminderAnnulus.tolerance_deg || any(isnan(gazePosition_deg))
        Screen('FrameOval',window,FixationReminderAnnulus.colour_rgb,FixationReminderAnnulus.rect,FixationReminderAnnulus.thickness_px); % draw annulus
        oms_drawfixation(window,ScreenInfo,CentralFixationTarget.size_deg,[0,0],CentralFixationTarget.colour,true,false,CentralFixationTarget.type,backgroundLuminance_AU); % draw fixation target
        [~,~,gazePosition_deg] = oms_eyetracker_getgazedata_latestsample(eye,tracker,ScreenInfo);
        keyPressed = oms_checkkeyboard;
        switch keyPressed
            case 'q'
                escapePressed = true;
                repeatTrial = false; % don't allow trial to be marked for repeating or we'll be stuck in an endless loop in the parent function
                return
        end
    end
    if CentralFixationTarget.on
        [~,fixationTargetRect] = oms_drawfixation(window,ScreenInfo,CentralFixationTarget.size_deg,[0,0],CentralFixationTarget.colour,true,true,CentralFixationTarget.type,backgroundLuminance_AU); % prepare central fixation target
    end
    Screen('Flip',window);
    WaitSecs(interstimulusInterval_sec); % give another wait period so the new stimulus doesn't appear immediately
end
fprintf('\nPresenting target at (%.1f,%.1f)',stimulusPosition_deg(1),stimulusPosition_deg(2))

if CentralFixationTarget.on
    [~,fixationTargetRect] = oms_drawfixation(window,ScreenInfo,CentralFixationTarget.size_deg,[0,0],CentralFixationTarget.colour,true,true,CentralFixationTarget.type,backgroundLuminance_AU); % prepare central fixation target
end

totalResponseWindow_sec = interstimulusInterval_sec - falsePositiveCatchTime_sec; % this is the total time we wait for keypresses

%% Present stimulus
stimulusOnsetTime_sec = GetSecs; % get the current time
stimulusOffsetTime_sec = stimulusOnsetTime_sec + stimulusDuration_ms/1000 - interFrameInterval/2; % stimulus is scheduled to disappear at the correct time minus half a monitor refresh (this makes sure it is scheduled for the exact monitor flip)
FlushEvents('keyDown');

keyPressed = []; % init
participantRespondedTooQuickly = false; % init
participantHasRespondedAppropriatelyToThisStimulus = false; % init
nKeyPresses = 0; % init
reactionTime_ms = NaN; % init

oms_eyetracker_sendmessage('Stimulus on') % send a message to the eyetracker to note when the stimulus first appears
oms_eyetracker_sendmessage(['Original stimulus position at ',num2str(stimulusPosition_deg(1)),',',num2str(stimulusPosition_deg(2)),' at time of presentation']) % send a message to eye tracker so we have a record of where the original stimulus presentation was
while GetSecs < stimulusOffsetTime_sec % while the stimulus is shown...
    [~,~,gazePosition_deg] = oms_eyetracker_getgazedata_latestsample(eye,tracker,ScreenInfo);
    switch gazeContingencyType
        case 'Continuous gaze-contingency (stimulus moving)'
            actualStimulusPosition_deg = [stimulusPosition_deg(1) + gazePosition_deg(1) + driftCorrectionOffset_deg(1) + manualFixationOffset_deg(1),stimulusPosition_deg(2) + gazePosition_deg(2) + driftCorrectionOffset_deg(2) + manualFixationOffset_deg(2)]; % get new position of gaze-contingent stimuli
        case 'No gaze-contingency'
            actualStimulusPosition_deg = stimulusPosition_deg + driftCorrectionOffset_deg + manualFixationOffset_deg;
    end
    actualStimulusPosition_px = oms_convert_degrees2px(actualStimulusPosition_deg,ScreenInfo);
    actualStimulusPosition_px(1) = actualStimulusPosition_px(1) + ScreenInfo.width_px/2; % add half-screen width as we are dealing with pixel coordinate system (origin in top-left)
    actualStimulusPosition_px(2) = -actualStimulusPosition_px(2) + ScreenInfo.height_px/2; % invert and translate (same reasons as above)
    actualStimulusRect = CenterRectOnPoint(StimulusList_main(stimulusIndex).rect,actualStimulusPosition_px(1),actualStimulusPosition_px(2)); % calculate actual stimulus rect (takking into account any calibration offset and gaze contingent corrections)
    
    if strcmp(gazeContingencyType,'Continuous gaze-contingency (stimulus moving)') && (any(UnionRect(ScreenInfo.rect,actualStimulusRect) ~= ScreenInfo.rect) || (CentralFixationTarget.on && any(ClipRect(actualStimulusRect,fixationTargetRect))) || any(isnan(actualStimulusRect))) % if this is a gaze contingent presentation but the stimulus was either partially off-screen, not shown at all, or overlapped with the fixation target...
        repeatTrial = true; % mark the trial as needing to repeat
        oms_eyetracker_sendmessage('Inappropriate stimulus position - trial will be repeated'); % send a message to eye tracker so we have a record of where the eye was at time of each presentation
        fprintf('\nInappropriate stimulus position - trial will be repeated')
    else % if this is non-gaze contingent or it is gaze-contingent and the eye is found
        stimulusActuallyPresented = true;
        if any(isnan(actualStimulusPosition_deg)) % if the eye is lost (note, this will only execute in the non-gaze contingent condition as we have already checked under the gaze-contingent condition above)
            oms_eyetracker_sendmessage('Eye position lost'); % make a note that the eye was lost, but carry on with presentation
        end
        Screen('FillOval',window,stimulusIntensity,actualStimulusRect); % draw the stimulus
        
        % Send messages and commands to the eyetracker:
        oms_eyetracker_sendmessage('clear_screen 0','command'); % clear eyetracker screen
        actualStimulusPosition_px = oms_convert_degrees2px(actualStimulusPosition_deg,ScreenInfo) + [ScreenInfo.width_px,ScreenInfo.height_px];
        oms_eyetracker_sendmessage(['draw_cross ',num2str(actualStimulusPosition_px(1)),' ',num2str(actualStimulusPosition_px(2)),' 15'],'command'); % draw representation on the Host PC window. The 15 means use colour #15. You can pick any colour from 0 to 15.
        oms_eyetracker_sendmessage(['Eye position at ',num2str(gazePosition_deg(1) + driftCorrectionOffset_deg(1) + manualFixationOffset_deg(1)),',',num2str(gazePosition_deg(2) + driftCorrectionOffset_deg(2) + manualFixationOffset_deg(2)),' at time of presentation']) % % send a message to eye tracker so we have a record of where the eye was at time of each presentation
        oms_eyetracker_sendmessage(['Stimulus position at ',num2str(actualStimulusPosition_deg(1)),',',num2str(actualStimulusPosition_deg(2)),' at time of presentation']) % send a message to eye tracker so we have a record of where the stimulus was at time of presentation
    end
    
    if CentralFixationTarget.on % redraw central fixation target during moving stimuli
        oms_drawfixation(window,ScreenInfo,CentralFixationTarget.size_deg,[0,0],CentralFixationTarget.colour,true,true,CentralFixationTarget.type,backgroundLuminance_AU); % prepare central fixation target
        oms_eyetracker_sendmessage(['draw_cross ',num2str(ScreenInfo.width_px/2),' ',num2str(ScreenInfo.height_px/2),' 15'],'command'); % draw a representation of the central fixation cross at the middle of the screen. The 15 means use colour #15. You can pick any colour from 0 to 15.
    end
    Screen('Flip',window);
    
    if CharAvail % if a key is pressed...
        nKeyPresses = nKeyPresses + 1; % keep track of how many button presses occurred
        if GetSecs - stimulusOnsetTime_sec < falsePositiveCatchTime_sec % ... and we're within the false positive window ('trigger happy')
            participantRespondedTooQuickly = true; % a false positive key has been pressed during the catch time
        else % ... if we're now past the false positive window
            participantHasRespondedAppropriatelyToThisStimulus = true; % need to set for later to check if a key was indeed pressed during the legitimate last 20 ms of the 200 ms stimulus duration
            keyPressed = char(GetChar(0,1)); % get the ASCII code of the key pressed
            reactionTime_ms = (GetSecs - stimulusOnsetTime_sec) * 1000;
        end
    end
end
if CentralFixationTarget.on % redraw central fixation target during moving stimuli
    oms_drawfixation(window,ScreenInfo,CentralFixationTarget.size_deg,[0,0],CentralFixationTarget.colour,true,true,CentralFixationTarget.type,backgroundLuminance_AU); % prepare central fixation target
    oms_eyetracker_sendmessage(['draw_cross ',num2str(ScreenInfo.width_px/2),' ',num2str(ScreenInfo.height_px/2),' 15'],'command'); % draw a representation of the central fixation cross at the middle of the screen. The 15 means use colour #15. You can pick any colour from 0 to 15.
end
Screen('Flip',window);
oms_eyetracker_sendmessage('Stimulus extinguished') % send message to eyetracker to record when the stimulus disappears from the screen

%% Get participant response (unless participant responded during presentation)
while GetSecs - stimulusOnsetTime_sec < falsePositiveCatchTime_sec % whilst we're still within the false positive window...
    if CharAvail
        nKeyPresses = nKeyPresses + 1; % keep track of how many button presses occurred
        participantRespondedTooQuickly = true; % a false positive key has been pressed during the catch time
    end
end

FlushEvents('keyDown'); % flush all keypresses from the cache
interStimulusInterval_sec = dynamicIntervalRange_min_max_sec(1) + rand * (dynamicIntervalRange_min_max_sec(2) - dynamicIntervalRange_min_max_sec(1)); % calculate a random delay before the next presentation
if ~participantRespondedTooQuickly && ~participantHasRespondedAppropriatelyToThisStimulus % if there hasn't been any response so far (neither appropriate nor inappropriate)
    while GetSecs - stimulusOnsetTime_sec < totalResponseWindow_sec % if the time is less than the interstimulus interval plus the random delay
        if CharAvail
            nKeyPresses = nKeyPresses + 1; % keep track of how many button presses occurred
            keyPressed = char(GetChar(0,1)); % get the ASCII code of the key pressed
            reactionTime_ms = (GetSecs - stimulusOnsetTime_sec) * 1000;
        end
    end
end

while GetSecs - stimulusOnsetTime_sec < totalResponseWindow_sec + interStimulusInterval_sec % now wait until the interStimulusInterval_sec has passed
    % Do nothing... Just wait until this time period has passed...
    if CharAvail % ... but check to see if a key was pressed in this time
        nKeyPresses = nKeyPresses + 1; % keep track of how many button presses occurred
    end
end

if participantRespondedTooQuickly % if the observer did respond within 180 ms of stimulus onset and a false positive was therefore counted, discard this stimulus presentation and set everything to blank, false and zero as the function needs these outputs even if they're empty
    reactionTime_ms = NaN;
end

% <The block below deals with *removing* the stimulus from the screen>
oms_eyetracker_sendmessage('clear_screen 0','command'); % clear Host PC representation
if repeatTrial
    return
end

% Store info about the trial:
reactionTimeRecord_sec(nPresentationsThisWave) = reactionTime_ms/1000; % record reaction time for presentation
StimulusList_main(stimulusIndex).nPresentations = StimulusList_main(stimulusIndex).nPresentations + 1; % tally how many times each stimulus has been presented
StimulusList_main(stimulusIndex).Presentations(StimulusList_main(stimulusIndex).nPresentations).reactionTime_ms = reactionTime_ms;
StimulusList_main(stimulusIndex).Presentations(StimulusList_main(stimulusIndex).nPresentations).trialType = trialType;
StimulusList_main(stimulusIndex).Presentations(StimulusList_main(stimulusIndex).nPresentations).stimulusIntensity = stimulusIntensity;
StimulusList_main(stimulusIndex).Presentations(StimulusList_main(stimulusIndex).nPresentations).timestamp = stimulusStartTimestamp;

if ~isempty(keyPressed)
    switch keyPressed
        case 'q'
            escapePressed = true;
            StimulusList_main(stimulusIndex).Presentations(StimulusList_main(stimulusIndex).nPresentations).seen = NaN;
            return
        case ' '
            StimulusList_main(stimulusIndex).Presentations(StimulusList_main(stimulusIndex).nPresentations).seen = true;
            fprintf(['\n',trialType,' seen'])
            if strcmp(trialType,'false positive') % if this is a false negative catch trial
                nFalsePositiveResponses = nFalsePositiveResponses + 1;
            end
        otherwise % if another button was pressed, but not any of the buttons we wanted...
            StimulusList_main(stimulusIndex).Presentations(StimulusList_main(stimulusIndex).nPresentations).seen = false; % count this as not seen...
    end
else
    StimulusList_main(stimulusIndex).Presentations(StimulusList_main(stimulusIndex).nPresentations).seen = false;
    fprintf(['\n',trialType,' not seen'])
    if strcmp(trialType,'false negative') % if this is a false negative catch trial
        nFalseNegativeResponses = nFalseNegativeResponses + 1;
    end
end

if ~stimulusActuallyPresented && StimulusList_main(stimulusIndex).Presentations(StimulusList_main(stimulusIndex).nPresentations).seen % if nothing was shown but participant responded as seen...
    nFalsePositiveResponses = nFalsePositiveResponses + 1;
end

if participantRespondedTooQuickly || nKeyPresses > 1
    nFalsePositiveResponses = nFalsePositiveResponses + 1;
end
